package com.zipcodewilmington.generic.group;

import com.zipcodewilmington.generic.identifiables.IdentifiableInterface;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.function.Predicate;
import java.util.stream.Collectors;

/**
 * @author leon on 06/12/2018.
 */
public class Group<TypeOfId extends Serializable,
        TypeOfEntity extends IdentifiableInterface<TypeOfId>> extends AbstractGroup<TypeOfId, TypeOfEntity>{

    public Group() {
    }

    @Override
    public Integer count() {
        return group.size();
    }

    @Override
    public void insert(IdentifiableInterface object) {
        group.add((TypeOfEntity) object);
    }

    @Override
    public void delete(IdentifiableInterface object) {
        group.remove(object);
    }

    @Override
    public void delete(Serializable serializable) {
        Iterator<TypeOfEntity> it = group.iterator();

        while(it.hasNext()) {
            TypeOfEntity entity = it.next();
            if (entity.getIdentity().equals(serializable))
                group.remove(entity);
        }
    }

    @Override
    public Boolean has(IdentifiableInterface object) {

        return group.contains(object);
    }

    @Override
    public Boolean has(Serializable serializable) {

        Iterator<TypeOfEntity> it = group.iterator();

        while(it.hasNext()) {
            TypeOfEntity entity = it.next();
            if (entity.getIdentity().equals(serializable))
                return true;
        }
        return false;
    }

    @Override
    public List<TypeOfEntity> filter(Predicate<TypeOfEntity> predicate) {

        return group.stream()
                .filter(predicate)
                .collect(Collectors.toList());
    }

    @Override
    public Class getIdentityType() {

        return group.get(0).getClass();
    }

    @Override
    public Class getIdentifiableType() {

        return group.get(0).getClass();
    }

}
